@@ -16,11 +16,16 @@ class OpenAIClient:
                  api_key="EMPTY",
                  concurrency_limit=10,
                  timeout=100,
+                 retry_times=3,
+                 retry_delay=0.55,
                  **kwargs):
         self._client = ConcurrentRequester(
             concurrency_limit=concurrency_limit,
             timeout=timeout,
+            retry_times=retry_times,
+            retry_delay=retry_delay,
         )
+        self._concurrency_limit = concurrency_limit
         self._headers = {"Content-Type": "application/json", "Authorization": f"Bearer {api_key}"}
         self._base_url = base_url.rstrip("/")
         self._api_key = api_key
@@ -41,25 +46,29 @@ class OpenAIClient:
         }
         return request_params
 
-    async def chat_completions(self, messages: list, model: str, return_raw=False, preprocess_msg=False, **kwargs) -> Union[str, "RequestResult"]:
+    async def chat_completions(self, messages: list, model: str, return_raw=False, preprocess_msg=False, url=None, **kwargs) -> Union[str, "RequestResult"]:
         result, _ =  await self._client.process_requests(
             request_params=[await self.wrap_to_request_params(messages, model, preprocess_msg=preprocess_msg, **kwargs)],
-            url=f"{self._base_url}/chat/completions",
+            url=url or f"{self._base_url}/chat/completions",
             method="POST",
             show_progress=False,
         )
+        data = result[0]
         if return_raw:
-            return result[0]
-        return result[0].data["choices"][0]["message"]["content"]
-
-    def chat_completions_sync(self, messages: list, model: str, return_raw=False, **kwargs):
-        return asyncio.run(self.chat_completions(messages, model, return_raw, **kwargs))
-
+            return data
+        else:
+            if data.status == "success":
+                return data.data["choices"][0]["message"]["content"]
+            else:
+                return data
 
+    def chat_completions_sync(self, messages: list, model: str, return_raw=False, url=None, **kwargs):
+        return asyncio.run(self.chat_completions(messages=messages, model=model, return_raw=return_raw, url=url, **kwargs))
 
     async def chat_completions_batch(self,
                                      messages_list: list[list],
                                      model: str,
+                                     url=None,
                                      return_raw=False,
                                      show_progress=True,
                                      preprocess_msg=False,
@@ -67,12 +76,12 @@ class OpenAIClient:
         if preprocess_msg:
             messages_list = await batch_process_messages(messages_list,
                                                          preprocess_msg=preprocess_msg,
-                                                         max_concurrent=self._client._concurrency_limit)
+                                                         max_concurrent=self._concurrency_limit)
 
         results, _ =  await self._client.process_requests(
             request_params=[await self.wrap_to_request_params(messages, model, preprocess_msg=False, **kwargs)
                             for messages in messages_list],
-            url=f"{self._base_url}/chat/completions",
+            url=url or f"{self._base_url}/chat/completions",
             method="POST",
             show_progress=show_progress,
         )
